
function data = mcreadgdif(streamID,dirpath)
%%
% Reads the data in a single stream from all sdif files in a directory
% 
% syntax 
% data = mcreadgdif
% data = mcreadgdif(streamID)
% data = mcreadgdif(streamID,path)
% 
% input parameters
% streamID: Which sdif stream to read into the mocap struct (default = 'XP30')
% path:     Which foder to search for sdif files in (default = current working directory)
%
% output:
% data: array of MoCap Structs
%
% Example:
% data = mcreadgdif('XPOS');
% data = mcreadgdif('XPOS',./datafiles);
% data = mcreadgdif('XPOS',/Users/Me/Documents/SdifFiles);
% data = mcreadgdif;
%
%
% NB! The MoCap toolbox assumes a fixed sampling rate.. make sure that your
% GDIF recording also has a fixed sampling rate. Otherwise, the estimated
% sampling rate will be incorrect.
% 
% The script tries to compensate for gaps in the data when estimating
% sampling rate. However, large amounts of jitter in the gdif file will
% cause problems...
%
% Easdif must be compiled and available in Matlab's search path for this to
% work

%various constants
trimtbs = 15;


%aruments, defaults etc.:
if nargin < 1
    streamID = 'XP30';
end
if nargin < 2
    dirpath = './';
    disp('Reading SDIF files in current directory')
else
    if dirpath(1) == '~'
        error('Please refrain from using the ''~'' character. Use the full path. (Will fix this later.)')
    end     
    if dirpath(length(dirpath)) ~= '/'
        dirpath(length(dirpath)+1) = '/';
    end 
    disp(['Reading SDIF files in ' dirpath])
end


%find all sdif files in directory:
a = dir([dirpath '*.sdif']);

%initiate mocap struct:
data = struct('type', {},'filename', {},'nFrames', {},'nCameras', {},'nMarkers', {},'freq', {},'nAnalog', {},'anaFreq', {},'timederOrder', {},'markerName', {},'data', {},'analogdata', {},'other', {});


%read sdif data into tmp variable:
for i = 1:length(a)
    [~, label] = fileparts(a(i).name);

   
    disp(['Reading:' label])

        
    data(i).filename  = [label '.sdif'];
    
    [tmp(i).file,tmp(i).head,tmp(i).dir] = Fsdifopen([dirpath  a(i).name]);
    [tmp(i).data,tmp(i).header,tmp(i).frame,tmp(i).matrix,tmp(i).dims] = Fsdifloadfile([dirpath  a(i).name]);
    Fsdifclose(tmp(i).file);
end



sound(0.3*sin((0:0.1:100)*pi),44100)
disp('done reading...')



disp('Putting SDIF data into MoCap structure...')
%put tmp data into mocap struct
for j=1:length(tmp)
    data(j).type  = 'MoCap data';
    
    timestamps = [];
    tmpdata = [];

    for i = 1:length(tmp(j).header)
       if strcmp(streamID,sprintf('%c%c%c%c',tmp(j).head.TYP.MTD(tmp(j).header(i,2)+1).msig)) 
           tmpdata(size(tmpdata,1)+1,1:(length(tmp(j).data{i}(:)))) = tmp(j).data{i}(:);
%           data(j).data(size(data(j).data,1)+1,1:(length(tmp(j).data{i}(:)))) = tmp(j).data{i}(:);
           timestamps(size(timestamps,1)+1,1) = tmp(j).header(i,1);
       end
    end
    
    if isempty(tmpdata)
        error([streamID ' not found. These are available; ' sprintf('%c%c%c%c ',tmp(j).head.TYP.MTD.msig) '.'])
    end
    
    %time between samples:
    tbs = filter([1 -1], 1,timestamps);
    sorttbs = sort(tbs);
    %remove high and low values from tbs before estimating sampling frequency (decreasing risk of errors due to gaps)
    data(j).freq = round(1/mean(sorttbs(round(length(sorttbs)/trimtbs):round(length(sorttbs)-length(sorttbs)/trimtbs))));
    samples = orgsamples(round(timestamps*data(j).freq));
    
    
    data(j).data(samples,:) = tmpdata;

    for n = 1:length(data(j).data(1,:))
        for m = 1:length(data(j).data(:,1))
            if data(j).data(m,n) == 0;
                data(j).data(m,n) = nan;
            end
        end
    end
    
    
    data(j).nFrames = length(data(j).data(:,1));
    
    
    if mod(length(data(j).data(1,:)),3) == 0
        data(j).nMarkers = length(data(j).data(1,:))/3;
    else
        error('marker streams must be 3D...')
    end
    
    data(j).nAnalog = 0;
    data(j).anaFreq = 0;
    data(j).timederOrder = 0;
    data(j).analogdata = 0;
    data(j).other.descr = 'MoCap struct containing GDIF data';
    data(j).other.dataIncluded = '3D';
    data(j).other.timestamp = a(j).date;

        
    sound(0.1*sin((0:(0.04+j/200):50)*pi),44100)
end

sound(0.3*sin((0:0.01:200).*(0.5:0.00005:1.5)).*sin(2:0.1:2002),44100)

end


function samples = orgsamples(badsamples)
    off = 1-badsamples(1);
    shift = 0;
    badsamples = badsamples+off;
    samples(1) = badsamples(1);
    
    for i = 2:length(badsamples)
        if badsamples(i) > samples(i-1)
            samples(i) = badsamples(i);
        else
            samples(i) = samples(i-1)+1;
        end
    end
end
